import { Component, Input } from '@angular/core';

interface Partition {
  size: number;
  type: string;
  sizeBytes: number;
  format: boolean;
  color: string;
  percentage: number;
}

@Component({
  selector: 'app-partition-assistant',
  templateUrl: './partition-assistant.component.html',
  styleUrls: ['./partition-assistant.component.css']
})
export class PartitionAssistantComponent {
  @Input() data: any;
  diskNumber = 1;
  partitionTable = 'MSDOS';
  totalDiskSize = 100;
  errorMessage = '';


  ngOnInit() { console.log('Data:', this.data); }

  partitions: Partition[] = [
    { size: 17.96, type: 'NTFS', sizeBytes: 17.96 * 1024 * 1024 * 1024, format: false, color: '#00a2e8', percentage: 17.96 },
    { size: 17.29, type: 'LINUX', sizeBytes: 17.29 * 1024 * 1024 * 1024, format: false, color: '#6a5acd', percentage: 17.29 },
    { size: 12.64, type: 'CACHE', sizeBytes: 12.64 * 1024 * 1024 * 1024, format: false, color: '#ff0000', percentage: 12.64 }
  ];

  addPartition() {
    const remainingGB = this.getRemainingGB();
    if (remainingGB > 0) {
      this.partitions.push({
        size: 0,
        type: 'NTFS',
        sizeBytes: 0,
        format: false,
        color: '#' + Math.floor(Math.random() * 16777215).toString(16),
        percentage: 0
      });
    } else {
      this.errorMessage = 'No hay suficiente espacio libre en el disco para crear una nueva partición.';
    }
  }

  removePartition(index: number) {
    this.partitions.splice(index, 1);
    this.updatePartitionPercentages();
  }

  updatePartitionSize(index: number, type: 'gb') {
    const partition = this.partitions[index];
    const remainingGB = this.getRemainingGB() + partition.size;
    
    if (partition.size > remainingGB) {
      this.errorMessage = `El tamaño de la partición no puede superar el espacio libre (${remainingGB.toFixed(2)} GB).`;
      partition.size = 0;
    } else {
      this.errorMessage = '';
      partition.sizeBytes = partition.size * 1024 * 1024 * 1024;
      this.updatePartitionPercentages();
    }
  }

  updatePartitionPercentages() {
    const totalUsedGB = this.partitions.reduce((acc, partition) => acc + partition.size, 0);
    const remainingGB = this.totalDiskSize - totalUsedGB;

    this.partitions.forEach(partition => {
      partition.percentage = (partition.size / this.totalDiskSize) * 100;
    });

    if (remainingGB > 0) {
      const freeSpaceIndex = this.partitions.findIndex(partition => partition.type === 'LIBRE');
      
      if (freeSpaceIndex > -1) {
        this.partitions[freeSpaceIndex].size = remainingGB;
        this.partitions[freeSpaceIndex].percentage = (remainingGB / this.totalDiskSize) * 100;
      } 
    } 
  }

  getRemainingGB(): number {
    const totalUsedGB = this.partitions.reduce((acc, partition) => acc + partition.size, 0);
    return Math.max(0, this.totalDiskSize - totalUsedGB);
  }

  save() {
    const totalSize = this.partitions.reduce((acc, partition) => acc + partition.size, 0);
  
    if (totalSize > this.totalDiskSize) {
      this.errorMessage = `El tamaño total de las particiones (${totalSize} GB) supera el tamaño total del disco (${this.totalDiskSize} GB).`;
      return;
    } else {
      this.errorMessage = '';
    }
  
    const payload = this.partitions.map((partition, index) => ({
      diskNumber: this.diskNumber,
      partitionNumber: index + 1,
      size: partition.size,
      filesystem: partition.type,
      format: partition.format
    }));
  
    console.log('Payload:', JSON.stringify(payload, null, 2));
  }
}
